@@ -263,7 +263,7 @@ def interpolate_field(data_dir, df, groups, grids, frame, field_values=['vx', 'v
         # interpolate vfield for each frame
         if frame_ in df['frame'].unique():
             group_ = groups.get_group(frame_).reset_index(drop=True)
-            no_nan_ = group_[np.isfinite(group_[coord_list])]
+            no_nan_ = group_[coord_list].dropna()
 
             if no_nan_.shape[0] > 2:  # cannot triangulate with less then 3 points
                 triang = tri.Triangulation(no_nan_['x'].values, no_nan_['y'].values)
@@ -281,7 +281,7 @@ def interpolate_field(data_dir, df, groups, grids, frame, field_values=['vx', 'v
     for coord in field_values:
         # average interpolated vfield over the frame list
         stack = np.ma.stack(field_dict[coord], axis=2)  # stack frames along axis 2
-        denom = len(frame_l) - np.sum(stack.mask, axis=2)  # denominator = number of non-masked data for each frame
+        denom = np.sum(~stack.mask, axis=2)  # denominator = number of non-masked data for each frame
         field = np.ma.filled(np.sum(stack, axis=2) / denom, np.nan)
         data[coord] = field
 
@@ -289,8 +289,9 @@ def interpolate_field(data_dir, df, groups, grids, frame, field_values=['vx', 'v
         export_dir = osp.join(outdir, 'field_export')
         tpr.safe_mkdir(export_dir)
         for k in data.keys():
-            f_out = osp.join(export_dir, k + '_t_{:04d}.txt'.format(int(frame)))
-            np.savetxt(f_out, data[k], delimiter=',')
+            if k != 'x' and k != 'y':  # don't save center grids because python path are not case sensitive
+                f_out = osp.join(export_dir, k + '_t_{:04d}.txt'.format(int(frame)))
+                np.savetxt(f_out, data[k], delimiter=',')
 
     return data
 
@@ -434,8 +435,10 @@ def compute_vector_mean(data_dir, data_dict, frame, field, dimensions=['x', 'y',
 
 def compute_all_vector_mean(data_dir, df, data_dict, field, dimensions=['x', 'y', 'z'], export_field=False,
                             outdir=None):
-    """Compute vector average defined as the modulus of the velocity or acceleration vector field, for all frames. Requires interpolated field data passed by data_dict.
-    The average can be in 2D or 3D."""
+    """
+    Compute vector average defined as the modulus of the velocity or acceleration vector field, for all frames. Requires interpolated field data passed by data_dict.
+    The average can be in 2D or 3D.
+    """
 
     if outdir is None:
         outdir = osp.join(data_dir, 'field')
@@ -454,7 +457,10 @@ def compute_all_vector_mean(data_dir, df, data_dict, field, dimensions=['x', 'y'
 
 
 def compute_vlim(df, data, field):
-    """Compute the min and max values (vlim) over all frames for a specific field"""
+    """
+    Compute the min and max values (vlim) over all frames for a specific field
+    """
+
     vlim = [None, None]
     for frame in df['frame'].unique():
         frame = int(frame)
