@@ -31,7 +31,7 @@ import copy
 
 from datetime import datetime
 
-
+from tqdm import tqdm
 import matplotlib.pyplot as plt
 import matplotlib as mpl
 import numpy as np
@@ -103,6 +103,9 @@ def plot_cmap(plot_dir, label, cmap, vmin, vmax, plot_config=None, suffix=''):
 
     plot_config = make_plot_config() if plot_config is None else plot_config
 
+    if vmin is None or vmax is None:  # can't plot a colormap with None boundary
+        return -1
+
     fig = plt.figure(figsize=(8, 3))
     ax = fig.add_axes([0.05, 0.80, 0.9, 0.15])
     norm = plt.Normalize(vmin=vmin, vmax=vmax)
@@ -115,7 +118,7 @@ def plot_cmap(plot_dir, label, cmap, vmin, vmax, plot_config=None, suffix=''):
 
 
 def plot_traj(df, frame, data_dir, groups=None, image={'image_fn': None, 't_dim': None, 'z_dim': None}, plot_dir=None,
-              show_plot=False, dim=3, plot_config=None,traj_parameters=None,quiet=0):
+              show_plot=False, dim=3, plot_config=None,traj_parameters=None):
     """ 
     Plot all trajectories of a given frame on an image if traj_parameters['no_bkg'] is False and an image is given.
     Plots can be color coded z value, by groups, or with random colors (traj_parameters['color_code']='z' or 'group' or 'random' or 'none')
@@ -123,10 +126,6 @@ def plot_traj(df, frame, data_dir, groups=None, image={'image_fn': None, 't_dim'
     It can be plotted in 3D with plot3D, elevation and angle set the 3D view
     """
 
-    if quiet < 1:
-        sys.stdout.write("\033[K")  # go back to previous line
-        print('plotting frame {}'.format(int(frame)), flush=True, end='\r')
-
     # get config parameters
     plot_config = make_plot_config() if plot_config is None else plot_config
 
@@ -197,7 +196,7 @@ def plot_traj(df, frame, data_dir, groups=None, image={'image_fn': None, 't_dim'
         else:
             colors = color_list[0]  # if color_coded by group but there's none, use only one color
     elif color_code == "random":
-        colors = [color_list[i % len(color_list)] for i in group['track'].values]
+        colors = [color_list[int(i) % len(color_list)] for i in group['track'].values]
     elif color_code == "none":
         colors = color_list[0]
     else:
@@ -233,7 +232,7 @@ def plot_traj(df, frame, data_dir, groups=None, image={'image_fn': None, 't_dim'
                     else:
                         colors = color_list[0]  # if color_coded by group but there's none, use only one color
                 elif color_code == "random":
-                    colors = color_list[track % len(color_list)]
+                    colors = color_list[int(track) % len(color_list)]
                 elif color_code == "none":
                     colors = color_list[0]
                 else:
@@ -254,7 +253,9 @@ def plot_traj(df, frame, data_dir, groups=None, image={'image_fn': None, 't_dim'
         return
 
     if show_axis:
-        fig.tight_layout()
+        ax.grid(False)
+        ax.patch.set_visible(False)
+        ax.set_position([0.125, 0.125, 0.9, 0.88])  # tight_layout doesn't work after add_axes is called in get_background()
 
     if save_as_stack: 
         return fig
@@ -266,13 +267,9 @@ def plot_traj(df, frame, data_dir, groups=None, image={'image_fn': None, 't_dim'
 
 def plot_scalar_field(data_dir, df, data, field, frame, image={'image_fn': None, 't_dim': None, 'z_dim': None},
                       map_param={'no_bkg': False, 'vlim': None, 'show_axis': False, 'cmap': 'plasma'},
-                      plot_dir=None, plot_config=None, quiet=0, dont_save=False):
+                      plot_dir=None, plot_config=None, dont_save=False):
     """Plot scalar field as colormap. The data needs to be generated before. """
 
-    if quiet < 1:
-        sys.stdout.write("\033[K")  # go back to previous line
-        print('plotting {} {}'.format(field, int(frame)), flush=True, end='\r')
-
     if plot_dir is None:
         plot_dir = osp.join(data_dir, field)
     tpr.safe_mkdir(plot_dir)
@@ -329,7 +326,7 @@ def plot_scalar_field(data_dir, df, data, field, frame, image={'image_fn': None,
     if show_axis:
         ax.grid(False)
         ax.patch.set_visible(False)
-        fig.set_tight_layout(True)
+        ax.set_position([0.125, 0.125, 0.9, 0.88])  # tight_layout doesn't work after add_axes is called in get_background()
 
     if invert_yaxis:
         ax.axis([xmin, xmax, ymax, ymin])
@@ -346,13 +343,9 @@ def plot_vector_field(data_dir, df, data, field, frame, plot_on_field=None, dim=
                       image={'image_fn': None, 't_dim': None, 'z_dim': None},
                       map_param={'no_bkg': False, 'vlim': None, 'show_axis': False, 'cmap': 'plasma',
                                  'size_factor': 1},
-                      plot_dir=None, plot_config=None, quiet=0):
+                      plot_dir=None, plot_config=None):
     """ Plot vector field"""
 
-    if quiet < 1:
-        sys.stdout.write("\033[K")  # go back to previous line
-        print('plotting {} {}'.format(field, int(frame)), flush=True, end='\r')
-
     if plot_dir is None:
         plot_dir = osp.join(data_dir, field)
     tpr.safe_mkdir(plot_dir)
@@ -387,8 +380,7 @@ def plot_vector_field(data_dir, df, data, field, frame, plot_on_field=None, dim=
             map_param_['vlim'] = plot_on_field['vlim']
             dim = 2  # to ensure that arrows are plotted in black and the z data is not use
             fig = plot_scalar_field(data_dir, df, data, plot_on_field['plot_on'], frame, image=image,
-                                        map_param=map_param_, plot_dir=plot_dir, plot_config=None,
-                                        quiet=2, dont_save=True)
+                                        map_param=map_param_, plot_dir=plot_dir, plot_config=None,dont_save=True)
             ax = fig.gca()
             invert_yaxis = False  # to ensure it's not inverted a second time
         else:
@@ -420,7 +412,7 @@ def plot_vector_field(data_dir, df, data, field, frame, plot_on_field=None, dim=
     if show_axis:
         ax.grid(False)
         ax.patch.set_visible(False)
-        fig.set_tight_layout(True)
+        ax.set_position([0.125, 0.125, 0.9, 0.88])  # tight_layout doesn't work after add_axes is called in get_background()
 
     if invert_yaxis:
         ylim = ax.get_ylim()
@@ -437,25 +429,33 @@ def plot_vector_field(data_dir, df, data, field, frame, plot_on_field=None, dim=
 def plot_Voronoi(data_dir, df, frame, data, show_local_area=True,
                  image={'image_fn': None, 't_dim': None, 'z_dim': None},
                  map_param={'no_bkg': False, 'vlim': None, 'show_axis': False,'cmap':'plasma','size_factor': 1,'line_width':1.},
-                 plot_dir=None, plot_config=None,quiet=0):
+                 plot_dir=None, plot_config=None):
     """
     Plot Voronoi tesselation and local area in 2D only.
-    :param data_dir:
-    :param df:
-    :param frame:
-    :param data:
-    :param show_local_area:
-    :param image:
+    :param data_dir: path to data directory
+    :type data_dir: str
+    :param df: tracks DataFrame
+    :type df: pandas.DataFrame
+    :param frame: frame to plot
+    :type frame: int
+    :param data: voronoi data computed by calculate.compute_Voronoi()
+    :type data: dict
+    :param show_local_area: plot area
+    :type show_local_area: bool
+    :param image: image information, output of prepare.get_image()
+    :type image: dict
     :param map_param:
-    :param plot_dir:
-    :param plot_config:
-    :return:
+    :type map_param: dict
+    :param plot_dir: path to plotting directory
+    :type plot_dir: str
+    :param plot_config: plot config output of make_plot_config
+    :type plot_config: dict or None
+    :param quiet: verbosity level
+    :type quiet: int
+    :return: figure is save_as_stack is True else None
+    :rtype: matplotlib.figure or None
     """
 
-    if quiet < 1:
-        sys.stdout.write("\033[K")  # go back to previous line
-        print('plotting {} {}'.format('voronoi', int(frame)), flush=True, end='\r')
-
     if plot_dir is None:
         plot_dir = osp.join(data_dir, 'voronoi')
     tpr.safe_mkdir(plot_dir)
@@ -514,7 +514,7 @@ def plot_Voronoi(data_dir, df, frame, data, show_local_area=True,
     if show_axis:
         ax.grid(False)
         ax.patch.set_visible(False)
-        fig.set_tight_layout(True)
+        ax.set_position([0.125, 0.125, 0.9, 0.88])  # tight_layout doesn't work after add_axes is called in get_background()
 
     if invert_yaxis:
         ylim = ax.get_ylim()
@@ -635,7 +635,7 @@ def plot_MSD(data_dir, track, track_groups=None, df=None, df_out=None, fit_model
 
 def plot_param_vs_param(data_dir, x_param, y_param, df=None, hue=None, hue_order=None, set_axis_lim=None,
                         plot_config=None, x_bin_num=None, ci=None, fit_reg=False, scatter=True,
-                        plot_dir=None, prefix='', suffix='', custom_var={}):
+                        plot_dir=None, prefix='', suffix='', custom_var={}, hue_palette=None):
     """Plot a parameter of df (y_param) against another parameter (x_param). Optional: compare datasets with hue as datasets identifier."""
 
     plot_config = make_plot_config() if plot_config is None else plot_config
@@ -694,10 +694,33 @@ def plot_param_vs_param(data_dir, x_param, y_param, df=None, hue=None, hue_order
             x_bins = np.linspace(df_[x_param].min(),df_[x_param].max(),x_bin_num+2)  # +2 so len(x_bins)=x_bin_num after removing min and max
             x_bins = x_bins[1:-1]
 
+        # hue config
+        leg = False  # dont legend by default
+        if hue is not None: 
+            df_[hue] = pd.to_numeric(df_[hue],errors='ignore')  # try to make it numeric if possible
+            if df_[hue].dtype.kind in 'iufc':  # if hue is numerical, dont plot legend, plot colormap instead
+                # plot colormap
+                if hue in custom_var.keys():
+                    label = tpr.make_param_label(None, manual_symbol=custom_var[hue]['name'], manual_unit=custom_var[hue]['unit'])
+                else: 
+                    label = tpr.make_param_label(hue, l_unit=info["length_unit"], t_unit=info["time_unit"])
+                plot_cmap(plot_dir, label, hue_palette, df_[hue].min(),
+                          df_[hue].max(),suffix='_scatterplot',plot_config=plot_config)
+                fit_reg = False  # dont fit 
+                hue_order = None # hue_order only works for categorical hue
+                palette = hue_palette 
+
+            else:  # if not numerical, display legend
+                leg = True
+                palette = color_list
+        else: 
+            leg = False
+            palette = None
+
         # plot
         g = sns.lmplot(x=x_param, y=y_param, hue=hue, hue_order=hue_order, data=df_, ci=ci, fit_reg=fit_reg, x_bins=x_bins, scatter=scatter,
-            facet_kws={'despine':plot_config['despine']})
-        if hue is not None:
+            facet_kws={'despine':plot_config['despine']}, legend=leg, palette=palette)
+        if leg:
             sns.move_legend(g, "right", frameon=False, title=None, bbox_to_anchor=(1.05, 0.5))
 
         fig = g.figure
@@ -708,13 +731,13 @@ def plot_param_vs_param(data_dir, x_param, y_param, df=None, hue=None, hue_order
         if set_axis_lim is not None:
             ax.set_xlim(set_axis_lim[0], set_axis_lim[1])
             ax.set_ylim(set_axis_lim[2], set_axis_lim[3])
-        else:  # recalculate because sometimes matplotlib auto limit fails
-            xlim_ = [df_[x_param].min(), df_[x_param].max()]
-            ylim_ = [df_[y_param].min(), df_[y_param].max()]
-            xlim_ = [xlim_[0] - 0.05 * (xlim_[1] - xlim_[0]), xlim_[1] + 0.05 * (xlim_[1] - xlim_[0])]
-            ylim_ = [ylim_[0] - 0.05 * (ylim_[1] - ylim_[0]), ylim_[1] + 0.05 * (ylim_[1] - ylim_[0])]
-            ax.set_xlim(xlim_[0], xlim_[1])
-            ax.set_ylim(ylim_[0], ylim_[1])
+        # else:  # recalculate because sometimes matplotlib auto limit fails
+        #     xlim_ = [df_[x_param].min(), df_[x_param].max()]
+        #     ylim_ = [df_[y_param].min(), df_[y_param].max()]
+        #     xlim_ = [xlim_[0] - 0.05 * (xlim_[1] - xlim_[0]), xlim_[1] + 0.05 * (xlim_[1] - xlim_[0])]
+        #     ylim_ = [ylim_[0] - 0.05 * (ylim_[1] - ylim_[0]), ylim_[1] + 0.05 * (ylim_[1] - ylim_[0])]
+        #     ax.set_xlim(xlim_[0], xlim_[1])
+        #     ax.set_ylim(ylim_[0], ylim_[1])
 
         if plot_config['despine']:
             sns.despine(fig)
@@ -790,8 +813,49 @@ def plot_param_hist(data_dir, param, df=None, hue=None, hue_order=None, hist=Tru
 
 def plot_param_boxplot(data_dir, df, x_param, param, order=None, hue=None, save_stat=False, hue_order=None,
                        boxplot=True, swarmplot=True, plot_config=None, plot_dir=None, prefix='', suffix='',
-                       leg_lab=None, custom_var={}):
-    """Plot boxplot between categories (given by x_param). Sub-categories can be plotted too (given by hue). A ttest can be performed has well"""
+                       leg_lab=None, custom_var={}, hue_swarm=None, hue_swarm_palette='plasma'):
+    """
+    Plot boxplot along categories (given by x_param). Sub-categories can be plotted too (given by hue).
+    Run statistical analysis (ttest) between categories.
+    :param data_dir: path to data directory
+    :type data_dir: str
+    :param df: data 
+    :type df: pandas.DataFrame
+    :param x_param: x parameter (categorical parameter)
+    :type x_param: str
+    :param param: y parameter (numerical parameter)
+    :type param: str
+    :param order: custom order for x_param values
+    :type order: list or None
+    :param hue: param to be used in boxplot hue
+    :type hue: str
+    :param save_stat: run ttest 
+    :type save_stat: bool
+    :param hue_order:
+    :type hue_order: list or None
+    :param boxplot: plot boxplot
+    :type boxplot: bool
+    :param swarmplot: plot swarmplot
+    :type swarmplot: bool
+    :param plot_config: plot config output of make_plot_config
+    :type plot_config: dict
+    :param plot_dir: path to plotting directory
+    :type plot_dir: str
+    :param prefix: filename prefix
+    :type prefix: str
+    :param suffix: filename suffix
+    :type suffix: str
+    :param leg_lab: legend label
+    :type leg_lab: str
+    :param custom_var: custom variable info
+    :type custom_var: dict
+    :param hue_swarm: parameter to be used in swarmplot hue
+    :type hue_swarm: str
+    :param hue_swarm_palette: colormap to be used for hue swarmplot
+    :type hue_swarm_palette: str
+    :return: None
+    """
+
 
     plot_config = make_plot_config() if plot_config is None else plot_config
     color_list = plot_config['color_list']
@@ -837,35 +901,69 @@ def plot_param_boxplot(data_dir, df, x_param, param, order=None, hue=None, save_
 
     # plotting
     if not dont_plot:
+        # hue config
+        if hue is None and hue_swarm is None:  # if no hue
+            width = 0.5  # make boxplot thinner
+            color = color_list[0] # use one color for boxplot
+            palette = [(0.25, 0.25, 0.25), (0.25, 0.25, 0.25)]  # use gray for swarmplot
+            dodge = False  # dont split swarmplot 
+        
+        elif hue is None and hue_swarm is not None: # if hue only for swarmplot
+            width = 0.5  # make boxplot thinner
+            color = None  # no color for boxplot
+            palette = hue_swarm_palette  # swarmplot colors
+            dodge = False  # dont split swarmplot            
+
+        elif hue is not None and hue_swarm is None: # if swarmplot hue same as boxplot hue
+            width = 0.8  # boxplot default width
+            color = None  # no color for boxplot
+            palette = color_list  # swarmplot colors
+            dodge = True  # split swarmplot  
+
+        elif hue is not None and hue_swarm is not None: # different hue between boxplot and swarmplot (this is ambiguous)
+            width = 0.8  # boxplot default width
+            color = None  # no color for boxplot
+            palette = hue_swarm_palette  # swarmplot colors
+            dodge = False  # dont split swarmplot 
+
+        # display hue_swarm legend only if not numerical
+        swarm_leg = False
+        if hue_swarm is not None: 
+            # if numerical, plot colormap instead
+            df_[hue_swarm] = pd.to_numeric(df_[hue_swarm],errors='ignore')  # make numeric if possible (so dtype isn't an object)
+            if df_[hue_swarm].dtype.kind in 'iufc': 
+                # plot colormap
+                if hue_swarm in custom_var.keys():
+                    label = tpr.make_param_label(None, manual_symbol=custom_var[hue_swarm]['name'], manual_unit=custom_var[hue_swarm]['unit'])
+                else: 
+                    label = tpr.make_param_label(hue_swarm, l_unit=info["length_unit"], t_unit=info["time_unit"])
+                plot_cmap(plot_dir, label, hue_swarm_palette, df_[hue_swarm].min(),
+                          df_[hue_swarm].max(),suffix='_swarmplot',plot_config=plot_config)
+            else:  # if not numerical, display legend
+                swarm_leg = "auto"
+
+        # plot figure
         fig, ax = plt.subplots(1, 1, figsize=figsize)
-        if hue is None:
-            width = 0.5
-            color = color_list[0]
-            palette = [(0.25, 0.25, 0.25), (0.25, 0.25, 0.25)]
-            lw = 0
-        else:
-            width = 0.8
-            color = None
-            palette = color_list
-            lw = 1
+
         if boxplot:
             sns.boxplot(data=df_, x=x_param, y=param, ax=ax, order=order, width=width, hue=hue, hue_order=hue_order,
                         color=color)
         if swarmplot:
-            sns.swarmplot(data=df_, x=x_param, y=param, ax=ax, order=order, size=8, linewidth=lw, dodge=True,
-                          palette=palette, hue=hue, hue_order=hue_order)
-        if hue is not None:
-            handles, labels = ax.get_legend_handles_labels()
-            if leg_lab is not None:
-                labs = leg_lab
-            else:
-                labs = labels[0:len(handles) // 2]
-            ax.legend(handles[0:len(handles) // 2], labs, frameon=False)
+            hue_ = hue_swarm if hue_swarm is not None else hue  # replace hue by hue_swarm
+            sns.swarmplot(data=df_, x=x_param, y=param, ax=ax, order=order, size=8, dodge=dodge,
+                          palette=palette, hue=hue_, hue_order=hue_order,legend=swarm_leg)
+        
+        # no frame around legend
+        if ax.get_legend() is not None:  # if legend exists
+            ax.legend(frameon=False)
+
+
         ax.ticklabel_format(axis='y', style='sci', scilimits=(-2, 3))
         ax.get_yaxis().get_major_formatter().set_useMathText(True)
         ax.set_ylabel(param_label)
         ax.set_xlabel(x_param)
 
+
         if plot_config['despine']:
             sns.despine(fig)
 
@@ -970,7 +1068,7 @@ def view_traj(df, image=None, z_step=1):
 
 
 def plot_all_traj(data_dir, df, image={'image_fn': None, 't_dim': None, 'z_dim': None}, parallelize=False, dim=3,
-                  plot_dir=None, traj_parameters=None, plot_config=None, quiet=0, custom_var={}):
+                  plot_dir=None, traj_parameters=None, plot_config=None, custom_var={}):
     """Plot traj for all frames"""
 
     # plotting directory
@@ -1042,11 +1140,14 @@ def plot_all_traj(data_dir, df, image={'image_fn': None, 't_dim': None, 'z_dim':
     else:
         stack = []  # stack to store images
         groups = df.groupby('frame')
-        for frame in np.sort(df['frame'].unique()):
+
+        frame_list = np.sort(df['frame'].unique())  # total frame list
+        frame_list_ = frame_list[::traj_parameters['increment']]  # frame list with potential non-1 increment
+        for frame in tqdm(frame_list_,desc='plotting trajectories',colour="green",file=sys.stdout,unit='frame'):
+            
             frame = int(frame)
             fig = plot_traj(df, frame, data_dir, groups=groups, image=image, plot_dir=plot_dir,
-                      traj_parameters=traj_parameters,
-                      dim=dim, plot_config=plot_config,quiet=quiet)
+                      traj_parameters=traj_parameters,dim=dim, plot_config=plot_config)
             
             # append to stack
             if save_as_stack:
@@ -1058,12 +1159,18 @@ def plot_all_traj(data_dir, df, image={'image_fn': None, 't_dim': None, 'z_dim':
 
         if save_as_stack:
             stack = np.array(stack)
-            tifff.imwrite(osp.join(plot_dir,'traj.tiff'), stack)
+            resolution = (1./info['lengthscale'], 1./info['lengthscale'])
+            labels = ['frame {}'.format(f) for f in frame_list_]
+            metadata = {'unit': info['length_unit'],
+                        'axes': 'TYXS',  # S axis is for RGB
+                        'Labels': labels,
+                        } 
+            tifff.imwrite(osp.join(plot_dir,'traj.tiff'), stack, imagej=True,resolution=resolution, metadata=metadata)
 
 
 def plot_all_scalar_fields(data_dir, df, data, field, image={'image_fn': None, 't_dim': None, 'z_dim': None},
                            map_param={'no_bkg': False, 'vlim': None, 'show_axis': False, 'cmap': 'plasma'},
-                           plot_dir=None, plot_config=None, quiet=0, custom_var={}):
+                           plot_dir=None, plot_config=None, custom_var={}):
     """Plot scalar fields as colormap for all frames."""
 
     # plotting directory
@@ -1079,6 +1186,10 @@ def plot_all_scalar_fields(data_dir, df, data, field, image={'image_fn': None, '
     # get vlim
     map_param_ = dict(map_param)
     vlim_default = tca.compute_vlim(df, data, field)
+    if field == 'div' or field == 'curl':  #  center vlim around zero
+        if vlim_default[0] is not None and vlim_default[1] is not None: 
+            abs_max_vlim = max(np.abs(vlim_default[0]),np.abs(vlim_default[1]))
+            vlim_default = [-abs_max_vlim,abs_max_vlim] 
 
     if map_param_['vlim'] is None:
         map_param_['vlim'] = vlim_default
@@ -1094,11 +1205,11 @@ def plot_all_scalar_fields(data_dir, df, data, field, image={'image_fn': None, '
     plot_cmap(plot_dir, label, map_param_['cmap'], map_param_['vlim'][0], map_param_['vlim'][1],suffix='_'+field,plot_config=plot_config)
 
     stack = []  # stack to store images
-    for frame in np.sort(df['frame'].unique()):
+    frame_list = np.sort(df['frame'].unique())  # total frame list
+    frame_list_ = frame_list[::map_param_['increment']]  # frame list with potential non-1 increment
+    for frame in tqdm(frame_list_,desc='plotting scalar field {}'.format(field),colour="green",file=sys.stdout,unit='frame'):
         frame = int(frame)
-        fig = plot_scalar_field(data_dir, df, data, field, frame, image=image,
-                                map_param=map_param_, plot_dir=plot_dir, 
-                                plot_config=plot_config, quiet=quiet)
+        fig = plot_scalar_field(data_dir, df, data, field, frame, image=image,map_param=map_param_, plot_dir=plot_dir, plot_config=plot_config)
         
         # append to stack
         if save_as_stack:
@@ -1110,15 +1221,20 @@ def plot_all_scalar_fields(data_dir, df, data, field, image={'image_fn': None, '
 
     if save_as_stack:
         stack = np.array(stack)
-        tifff.imwrite(osp.join(plot_dir, field +'.tiff'), stack)
-
+        resolution = (1./info['lengthscale'], 1./info['lengthscale'])
+        labels = ['frame {}'.format(f) for f in frame_list_]
+        metadata = {'unit': info['length_unit'],
+                    'axes': 'TYXS',  # S axis is for RGB
+                    'Labels': labels,
+                    }  
+        tifff.imwrite(osp.join(plot_dir, field +'.tiff'), stack, imagej=True,resolution=resolution, metadata=metadata)
 
 
 def plot_all_vector_fields(data_dir, df, data, field, plot_on_field=None, dim=3,
                            image={'image_fn': None, 't_dim': None, 'z_dim': None},
                            map_param={'no_bkg': False, 'vlim': None, 'show_axis': False, 'cmap': 'plasma',
                                       'size_factor': 1},
-                           plot_dir=None, plot_config=None, quiet=0, custom_var={}):
+                           plot_dir=None, plot_config=None, custom_var={}):
     """Plot vector fields for all frames."""
 
     # plotting directory
@@ -1151,10 +1267,12 @@ def plot_all_vector_fields(data_dir, df, data, field, plot_on_field=None, dim=3,
             plot_cmap(plot_dir, label, plot_on_field['cmap'], plot_on_field['vlim'][0], plot_on_field['vlim'][1],suffix='_vector_'+plot_on_field['plot_on'],plot_config=plot_config)
 
     stack = []  # stack to store images
-    for frame in np.sort(df['frame'].unique()):
+    frame_list = np.sort(df['frame'].unique())  # total frame list
+    frame_list_ = frame_list[::map_param['increment']]  # frame list with potential non-1 increment
+    for frame in tqdm(frame_list_,desc='plotting vector field {}'.format(field),colour="green",file=sys.stdout,unit='frame'):
         frame = int(frame)
         fig = plot_vector_field(data_dir, df, data, field, frame, plot_on_field=plot_on_field, dim=dim, image=image,
-                          map_param=map_param, plot_dir=plot_dir, plot_config=plot_config, quiet=quiet)
+                          map_param=map_param, plot_dir=plot_dir, plot_config=plot_config)
         
         # append to stack
         if save_as_stack:
@@ -1166,11 +1284,18 @@ def plot_all_vector_fields(data_dir, df, data, field, plot_on_field=None, dim=3,
 
     if save_as_stack:
         stack = np.array(stack)
-        tifff.imwrite(osp.join(plot_dir,'vector_' + field +'.tiff'), stack)
+        resolution = (1./info['lengthscale'], 1./info['lengthscale'])
+        labels = ['frame {}'.format(f) for f in frame_list_]
+        metadata = {'unit': info['length_unit'],
+                    'axes': 'TYXS',  # S axis is for RGB
+                    'Labels': labels,
+                    } 
+        tifff.imwrite(osp.join(plot_dir,'vector_' + field +'.tiff'), stack, imagej=True,resolution=resolution, metadata=metadata)
+
 
 def plot_all_Voronoi(data_dir, df, data, show_local_area=True, df_mean = None,
                      image={'image_fn': None, 't_dim': None, 'z_dim': None},
-                     map_param=None,plot_dir=None, plot_config=None,quiet=0):
+                     map_param=None,plot_dir=None, plot_config=None):
     """
     Plot Voronoi for all frames and calculate voronoi cell area.
     """
@@ -1223,10 +1348,12 @@ def plot_all_Voronoi(data_dir, df, data, show_local_area=True, df_mean = None,
         plot_cmap(plot_dir, label, map_param['cmap'], map_param['vlim'][0], map_param['vlim'][1],suffix='_area',plot_config=plot_config)
 
     stack = []  # stack to store images
-    for frame in np.sort(df['frame'].unique()):
+    frame_list = np.sort(df['frame'].unique())  # total frame list
+    frame_list_ = frame_list[::map_param['increment']]  # frame list with potential non-1 increment
+    for frame in tqdm(frame_list_,desc='plotting voronoi',colour="green",file=sys.stdout,unit='frame'):
         frame = int(frame)
         fig = plot_Voronoi(data_dir, df, frame, data, show_local_area=show_local_area, image=image, map_param=map_param,
-                     plot_dir=plot_dir, plot_config=plot_config, quiet=quiet)
+                     plot_dir=plot_dir, plot_config=plot_config)
 
         # append to stack
         if save_as_stack:
@@ -1238,7 +1365,13 @@ def plot_all_Voronoi(data_dir, df, data, show_local_area=True, df_mean = None,
 
     if save_as_stack:
         stack = np.array(stack)
-        tifff.imwrite(osp.join(plot_dir,'voronoi.tiff'), stack)
+        resolution = (1./info['lengthscale'], 1./info['lengthscale'])
+        labels = ['frame {}'.format(f) for f in frame_list_]
+        metadata = {'unit': info['length_unit'],
+                    'axes': 'TYXS',  # S axis is for RGB
+                    'Labels': labels,
+                    } 
+        tifff.imwrite(osp.join(plot_dir,'voronoi.tiff'), stack, imagej=True,resolution=resolution, metadata=metadata)
 
 
 def plot_all_MSD(data_dir, df=None, df_out=None, fit_model="biased_diff", msd_all=None, refresh=False, hue=None,
@@ -1478,6 +1611,7 @@ def plot_total_traj(data_dir, df, dim=3, plot_dir=None, plot_fn=None, plot_confi
     fig, ax = plt.subplots(1, 1, figsize=plot_config['figsize'])
     for tgv in track_groups_values:
         track = tgv[1] if type(tgv) is tuple else tgv
+        track = int(track)  # ensure track is integer
         traj = tpr.get_traj(track_groups, tgv)
         traj_length = traj.shape[0]
         first_frame_id = traj['frame'].idxmin()
