@@ -39,6 +39,9 @@ from skimage.color import rgb2gray
 from skimage.util import img_as_ubyte
 import tifffile as tifff
 
+from shapely.geometry import Point
+from shapely.geometry.polygon import Polygon
+
 from track_analyzer import plotting as tpl
 from track_analyzer import calculate as tca
 
@@ -724,13 +727,17 @@ def get_data(data_dir, df=None, refresh=False, split_traj=False, set_origin_=Fal
             df[d] = df[d].astype(np.float)
 
         # rename track id and deal with gaps
+        print("renaming tracks...")
+        df = df[df['track'].notna()]  # remove Nan and None tracks
         df = tca.regularize_traj(df, dimensions, split_traj)
 
         # scale data
+        print("applying length and time scales...")
         tca.scale_dim(df, dimensions=dimensions, timescale=timescale, lengthscale=lengthscale, z_step=z_step,
                       unit=table_unit, invert_axes=invert_axes)
 
         # compute velocities and accelerations
+        print("computing velocities and accelerations...")
         tca.compute_vel_acc(df, dimensions=dimensions, timescale=timescale)
 
         # reset coordinates origin
@@ -853,7 +860,62 @@ def filter_by_frame_subset(df, frame_subset=None):
     return df_
 
 
-def filter_by_region(df, xlim=None, ylim=None, zlim=None):
+def filter_by_region(df, xlim=None, ylim=None, zlim=None, polygon=None):
+    """
+    Filter data based on coordinates given in px. xy coordinates can be given by a rectangle (xlim,ylim) or a polygon.
+    Rectangle selection is default
+    :param df: dataframe of trajectories
+    :type df: pandas.DataFrame
+    :param xlim: [xmin,xmax]
+    :type xlim: list or None
+    :param ylim: [ymin,ymax]
+    :type ylim: list or None
+    :param zlim: [zmin,zmax]
+    :type zlim: list or None
+    :param polygon: polygon coordinates array(x_array,y_array)
+    :type polygon: numpy.array or None
+    :return: filtered dataframe of trajectories
+    :rtype: pandas.DataFrame
+    """
+
+    # make rectangle selection default
+    if (xlim is not None or ylim is not None) and polygon is not None: 
+        polygon = None 
+
+    # don't do anything if empty df
+    if df.shape[0] == 0:
+        return df
+
+    df_ = df.copy()
+
+    # filter by rectangle
+    if xlim is not None or ylim is not None:
+        df_ = filter_by_xyz(df_, xlim=xlim, ylim=ylim, zlim=zlim)
+
+    # filter by polygon
+    elif polygon is not None: 
+        # first filter by xy extrema to speed up the process
+        df_ = filter_by_xyz(df_, xlim=[polygon[:,0].min(),polygon[:,0].max()], ylim=[polygon[:,1].min(),polygon[:,1].max()], zlim=zlim)
+
+        # then refine filtering
+        poly_ = Polygon(polygon)
+        data = []
+
+        for ind in df_.index: 
+            pt = Point(df_.loc[ind,['x','y']].values)
+            if poly_.contains(pt):
+                data.append(dict(zip(df_.columns,df_.loc[ind,:].values)))
+        
+        df_ = pd.DataFrame(data)
+    
+    # if all filters are None
+    else: 
+        pass
+
+    return df_
+
+
+def filter_by_xyz(df, xlim=None, ylim=None, zlim=None):
     """
     Extract data within a box given by xlim, ylim and zlim in px
     :param df: dataframe of trajectories
@@ -897,7 +959,6 @@ def filter_by_region(df, xlim=None, ylim=None, zlim=None):
 def get_coordinates(image, df=None, verbose=True):
     """
     Interactive selection of coordinates on an image by hand-drawing using a Napari viewer.
-    To be used with get_coordinates
     :param image: dict returned by get_image()
     :type image: dict
     :param df: pd.dataframe of tracks
@@ -968,19 +1029,22 @@ def get_coordinates(image, df=None, verbose=True):
 
     # inspect selected layers
     rectangle_list = []
+    polygon_list = []
     if len(shape_list) > 0:
         for i, shape_type_ in enumerate(shape_list[0].shape_type):  # grab the first element to ignore if Enter is pressed several times
             if shape_type_ == 'rectangle':
                 rectangle_list.append(shape_list[0].data[i])
+            elif shape_type_ == 'polygon':
+                polygon_list.append(shape_list[0].data[i])
     points = np.array([])
     if len(points_list) > 0:
         points = points_list[0]  # grab the first element to ignore if Enter is pressed several times
     
     if verbose:
-        print('You have selected {} point(s) and {} rectangle(s)'.format(points.shape[0], len(rectangle_list)))
+        print('You have selected {} point(s), {} rectangle(s) and {} polygon(s)'.format(points.shape[0], len(rectangle_list), len(polygon_list)))
 
     # retreive coordinates
-    coord_dict = {'points': [], 'rectangle': []}
+    coord_dict = {'points': [], 'rectangle': [], 'polygon': []}
     
     # get rectangle coordinates
     for rect in rectangle_list:
@@ -1005,6 +1069,31 @@ def get_coordinates(image, df=None, verbose=True):
 
         coord_dict['rectangle'].append({'frame': frame, 'z': z, 'xlim': [xmin, xmax], 'ylim': [ymin, ymax]})
 
+    # get polygon coordinates
+    for poly in polygon_list:
+        # if 4D stack
+        if t_dim is not None and z_dim is not None:
+            frame = int(poly[0, t_dim])
+            z = int(poly[0, z_dim])
+            x_array = poly[:, 3]
+            y_array = poly[:, 2]
+        # if 3D (3rd dim being time or z)
+        elif t_dim is not None or z_dim is not None:
+            frame = int(poly[0, t_dim]) if t_dim is not None else None
+            z = int(poly[0, z_dim]) if z_dim is not None else None
+            x_array = poly[:, 2]
+            y_array = poly[:, 1]
+        # if 2D
+        else:
+            frame = None
+            z = None
+            x_array = poly[:, 1]
+            y_array = poly[:, 0]
+
+        coord = np.array((x_array,y_array)).T  # column vectors of coordinates
+
+        coord_dict['polygon'].append({'frame': frame, 'z': z, 'coord': coord})
+
     # get points coordinates
     for i in range(points.shape[0]):
         # if 4D stack
@@ -1057,9 +1146,10 @@ def filter_by_traj_id(df, track_list=None):
     return out_df
 
 
-def select_traj_by_xyzt(df, xlim=None, ylim=None, zlim=None, frame_lim=None):
+def select_traj_by_xyzt(df, xlim=None, ylim=None, zlim=None, polygon=None, frame_lim=None):
     """
-    Get ids of trajectories going through an xyzt box. The spatiotemporal box is defined its boundaries
+    Get ids of trajectories going through an xyzt box. The spatiotemporal box is defined its boundaries. 
+    Alternativaly xy coordinates can be given by a polygon coordinates. 
     :param df: dataframe of trajectories
     :type df: pandas.DataFrame
     :param xlim: x boundaries
@@ -1068,22 +1158,28 @@ def select_traj_by_xyzt(df, xlim=None, ylim=None, zlim=None, frame_lim=None):
     :param ylim: list or None
     :param zlim: z boundaries
     :param zlim: list or None
+    :param polygon: polygon coordinates
+    :param polygon: numpy.array or None
     :param frame_lim: frame boundaries or unique frame
     :param frame_lim: int or list or None
     :return: list of trajectories id
     :rtype: list
     """
+
+    # work on a copy
+    df_ = df.copy()
+
     # filter by frame subset
     if type(frame_lim) is list or type(frame_lim) is tuple:
-        df = filter_by_frame_subset(df, frame_subset=frame_lim)
+        df_ = filter_by_frame_subset(df_, frame_subset=frame_lim)
     elif type(frame_lim) is int or type(frame_lim) is float:
-        df = df[df['frame'] == frame_lim]
+        df_ = df_[df_['frame'] == frame_lim]
 
     # filter by region
-    df = filter_by_region(df, xlim=xlim, ylim=ylim, zlim=zlim)
+    df_ = filter_by_region(df_, xlim=xlim, ylim=ylim, zlim=zlim, polygon=polygon)
 
     # get ids
-    track_list = df['track'].unique()
+    track_list = df_['track'].unique()
     return track_list
 
 
@@ -1147,15 +1243,19 @@ def select_sub_data(df, filters=[]):
     subset_names = []
     for i, filt in enumerate(filters):
         # filter
-        df_ = filter_by_region(df, xlim=filt['xlim'], ylim=filt['ylim'], zlim=filt['zlim'])
+        df_ = filter_by_region(df, xlim=filt['xlim'], ylim=filt['ylim'], zlim=filt['zlim'], polygon=filt['ROI'])
         df_ = filter_by_frame_subset(df_, frame_subset=filt['frame_subset'])
         df_ = filter_by_traj_len(df_, min_traj_len=filt['min_traj_len'], max_traj_len=filt['max_traj_len'])
         df_ = filter_by_traj_id(df_, filt['track_list'])
         if filt['track_ROI'] is not None:
             track_list = select_traj_by_xyzt(df_,xlim=filt['track_ROI']['xlim'], ylim=filt['track_ROI']['ylim'], 
-                                            zlim=filt['track_ROI']['zlim'], frame_lim=filt['track_ROI']['frame_lim'])
+                                            zlim=filt['track_ROI']['zlim'], polygon=filt['track_ROI']['ROI'],
+                                            frame_lim=filt['track_ROI']['frame_lim'])
             df_ = filter_by_traj_id(df_, track_list)
 
+            # filter again by traj length because selected trajectories can be smaller
+            df_ = filter_by_traj_len(df_, min_traj_len=filt['min_traj_len'], max_traj_len=filt['max_traj_len'])
+
         if len(filters) > 1:  # populate subset_names only if there are several subset
             subset_name = filt['name'] if filt['name'] != '' else 'subset_{}'.format(i)  # subset name
 
@@ -1202,6 +1302,7 @@ def get_background(image=None, frame=None, df=None, no_bkg=False, image_size=Non
                 print("WARNING: no image nor data provided")
                 figsize = figsize
             else:
+                # define image size as slightly larger than the data range
                 xmin = df['x'].min() - 0.05 * (df['x'].max() - df['x'].min())
                 xmax = df['x'].max() + 0.05 * (df['x'].max() - df['x'].min())
                 ymin = df['y'].min() - 0.05 * (df['y'].max() - df['y'].min())
@@ -1360,6 +1461,7 @@ def make_map_config(data_dir=None, export_config=True, empty_run=True):
                  'export_field': False,  # export field points to txt files
                  'temporal_average': 0,  # number of frame to average map values over
                  'cmap': "plasma",  # color map for scalar fields
+                 'increment': 1,  # increment to plot frames 
                  }
 
     # config of scalar field to plot. Each key is a parameter to plot and it stores a dict containing the plotting config for this specific parameter
@@ -1436,6 +1538,7 @@ def make_traj_config(data_dir=None, export_config=True):
                     'elevation': None,  # 3D paramater !! Not supportes anymore !!
                     'angle': None,  # 3D paramater !! Not supportes anymore !!
                     'subset_order': None,  # if color_code is group, order of group in color cycle
+                    'increment': 1,  # increment to plot frames 
                     }
 
     MSD_config = {'run': True,  # run plot_all_MSD
@@ -1452,7 +1555,11 @@ def make_traj_config(data_dir=None, export_config=True):
 
     scatter_config = {'run': True,  # run param_vs_param
                       'couple_list': [['x', 'v'], ['y', 'v']],  # list of [x,y] couples of variables to be plotted in scatter
+                      'hue_var_list': [None,None],  # list of variables used for hue
+                      'hue_cmap_list': [None,None], # list of colormaps used for hue
                       'mean_couple_list': [['x', 'v'], ['y', 'v']],  # list of [x,y] couples of variables averaged along the whole track to be plotted in scatter
+                      'mean_hue_var_list': [None,None],  # list of variables used for hue
+                      'mean_hue_cmap_list': [None,None], # list of colormaps used for hue
                       'x_bin_num': None,  #  number of evenly spaced bins along x axis
                       'ci': None,  # show confidence interval [0,100]
                       'fit_reg': False,  # show regression fit
@@ -1466,10 +1573,14 @@ def make_traj_config(data_dir=None, export_config=True):
 
     boxplot_config = {'run': True,  # run plot_param_hist
                        'var_list': ['v'],  # list of variables to be plotted 
-                       'mean_var_list': ['v'],  # list of variables averaged along the whole track to be plotted
+                       'hue_var_list': [None],  # list of variables used for hue in swarmplots
+                       'hue_cmap_list': [None], # list of colormaps used for hue in swarmplots
                        'save_stat': True ,  # run statistical test (scipy.stats.ttest_ind(equal_var=False))
                        'boxplot': True,  # show boxplot for variables in var_list
                        'swarmplot': False,  # show swarmplot for variables in var_list
+                       'mean_var_list': ['v'],  # list of variables averaged along the whole track to be plotted
+                       'mean_hue_var_list': [None],  # list of mean variables used for hue in swarmplots
+                       'mean_hue_cmap_list': [None], # list of colormaps used for hue in swarmplots
                        'mean_boxplot': True,  # show boxplot for variables in mean_var_list
                        'mean_swarmplot': True,  # show swarmplot for variables in mean_var_list
                        'mean_save_stat': True ,  # run statistical test (scipy.stats.ttest_ind(equal_var=False)) for wholetrack parameters
@@ -1501,6 +1612,7 @@ def make_traj_config(data_dir=None, export_config=True):
                 'size_factor': 1.,  # to multiply the default size of lines !! Not implemented yet !!
                 'show_axis': False,  # to show the plot axes (by default just image)
                 'line_width': 1.,  # diagram line width
+                'increment': 1,  # increment to plot frames 
                }
 
     # package all in a dict
@@ -1571,6 +1683,7 @@ def init_filters(data_dir=None, export_config=False):
     subset_dict = {'xlim': None,  # x boundaries
                    'ylim': None,  # y boundaries
                    'zlim': None,  # z boundaries
+                   'ROI': None,  # polygon coordinates of selected region
                    'min_traj_len': None,  # minimum trajectory length
                    'max_traj_len': None,  # maximum trajectory length
                    'frame_subset': None,  # frame boundaries
