@@ -146,7 +146,7 @@ def map_analysis(data_dir, data=None, image=None, refresh=False, parallelize=Fal
         dir_name_ = '{}{}'.format(len(tpr.listdir_nohidden(map_dir)) + 1, dir_name)
 
         if quiet < 2:
-            print(r"Analyzing subset {}  ".format(i + 1, dir_name_))
+            print(r"Analyzing subset #{}, named: {}".format(i + 1, dir_name_))
         sub_dir = osp.join(map_dir, dir_name_)
         sub_dir = sub_dir + '_1' if osp.exists(sub_dir) else sub_dir  # dont overwrite existing dir
         tpr.safe_mkdir(sub_dir)
@@ -209,34 +209,35 @@ def map_analysis(data_dir, data=None, image=None, refresh=False, parallelize=Fal
                                                          export_field=map_param['export_field'], outdir=sub_dir)
 
         # plot data
-        if quiet < 2:
-            print("Plotting scalar fields...")
         scalar_fields_ = {**scalar_fields, **vector_mean}  # merge scalar data in one single dict
-        for field in scalar_fields_.keys():
-            if plot_config['save_as_stack']:
-                plot_dir = sub_dir
-            else: # save series of images in a separate directory 
-                plot_dir = osp.join(sub_dir, field)
-                tpr.safe_mkdir(plot_dir)
-            map_param_ = dict(map_param)
-            map_param_['vlim'] = scalar_fields_[field]['vlim'] if 'vlim' in scalar_fields_[field].keys() else None
-            map_param_['cmap'] = scalar_fields_[field]['cmap'] if 'cmap' in scalar_fields_[field].keys() else 'plasma'
-            tpl.plot_all_scalar_fields(data_dir, df, field_data, field, image=image, map_param=map_param_,
-                                       plot_dir=plot_dir, plot_config=plot_config, quiet=quiet, custom_var=custom_var)
+        if len(scalar_fields_.keys()) > 0:
+            if quiet < 2:
+                print("Plotting scalar fields...")
+            for field in scalar_fields_.keys():
+                if plot_config['save_as_stack']:
+                    plot_dir = sub_dir
+                else: # save series of images in a separate directory 
+                    plot_dir = osp.join(sub_dir, field)
+                    tpr.safe_mkdir(plot_dir)
+                map_param_ = dict(map_param)
+                map_param_['vlim'] = scalar_fields_[field]['vlim'] if 'vlim' in scalar_fields_[field].keys() else None
+                map_param_['cmap'] = scalar_fields_[field]['cmap'] if 'cmap' in scalar_fields_[field].keys() else 'plasma'
+                tpl.plot_all_scalar_fields(data_dir, df, field_data, field, image=image, map_param=map_param_,
+                                           plot_dir=plot_dir, plot_config=plot_config, custom_var=custom_var)
         
-        if quiet < 2:
-            print("Plotting vector fields...")
-        for field in vector_fields.keys():
-            if plot_config['save_as_stack']:
-                plot_dir = sub_dir
-            else: # save series of images in a separate directory 
-                plot_dir = osp.join(sub_dir, field)
-                tpr.safe_mkdir(plot_dir)
-            map_param_ = dict(map_param)
-            map_param_['vlim'] = vector_fields[field]['vlim']
-            tpl.plot_all_vector_fields(data_dir, df, field_data, field, image=image, plot_on_field=vector_fields[field],
-                                       dim=3, map_param=map_param_, plot_dir=plot_dir, plot_config=plot_config,
-                                       quiet=quiet, custom_var=custom_var)
+        if len(vector_fields.keys()) > 0:
+            if quiet < 2:
+                print("Plotting vector fields...")
+            for field in vector_fields.keys():
+                if plot_config['save_as_stack']:
+                    plot_dir = sub_dir
+                else: # save series of images in a separate directory 
+                    plot_dir = osp.join(sub_dir, field)
+                    tpr.safe_mkdir(plot_dir)
+                map_param_ = dict(map_param)
+                map_param_['vlim'] = vector_fields[field]['vlim']
+                tpl.plot_all_vector_fields(data_dir, df, field_data, field, image=image, plot_on_field=vector_fields[field],
+                                           dim=3, map_param=map_param_, plot_dir=plot_dir, plot_config=plot_config,custom_var=custom_var)
 
     return df_list
 
