@@ -167,6 +167,7 @@ def traj_analysis(data_dir, data=None, image=None, refresh=False, parallelize=Fa
                 hue = 'subset'
                 hue_order = df['subset'].unique() if filters['subset_order'] is None else filters['subset_order']
                 traj_config_['subset_order'] = hue_order
+                total_traj_config['subset_order'] = hue_order
             else:
                 hue = None
                 hue_order = None
@@ -179,7 +180,7 @@ def traj_analysis(data_dir, data=None, image=None, refresh=False, parallelize=Fa
             if quiet < 2:
                 print("Plotting trajectories...")
             tpl.plot_all_traj(data_dir, df, image=image, traj_parameters=traj_config_, parallelize=parallelize,
-                              dim=dim, plot_dir=sub_dir, plot_config=plot_config, quiet=quiet, custom_var=custom_var)
+                              dim=dim, plot_dir=sub_dir, plot_config=plot_config, custom_var=custom_var)
 
         if total_traj_config['run']:
             if quiet < 2:
@@ -215,7 +216,7 @@ def traj_analysis(data_dir, data=None, image=None, refresh=False, parallelize=Fa
                     plot_dir = osp.join(sub_dir, 'voronoi')
                     tpr.safe_mkdir(plot_dir)
                 tpl.plot_all_Voronoi(data_dir, df, vor_data, show_local_area=voronoi_config['show_local_area'], image=image,
-                                     map_param=voronoi_config, plot_dir=plot_dir, plot_config=plot_config, quiet=quiet)
+                                     map_param=voronoi_config, plot_dir=plot_dir, plot_config=plot_config)
 
         if hist_config['run']:
             fn = osp.join(config_dir, 'hist_config.csv')
@@ -245,21 +246,23 @@ def traj_analysis(data_dir, data=None, image=None, refresh=False, parallelize=Fa
                 if len(scatter_config['couple_list']) > 0:
                     if quiet < 2:
                         print("Plotting scatter plots of instantaneous parameters...")
-                for param_vs_param in scatter_config['couple_list']:
+                for i,param_vs_param in enumerate(scatter_config['couple_list']):
                     x_param, y_param = param_vs_param
+                    hue_ = scatter_config['hue_var_list'][i] if hue is None else hue  # if hue not already defined by dataset, replace by specific hue
                     tpl.plot_param_vs_param(data_dir, x_param, y_param, df, plot_dir=sub_dir, plot_config=plot_config,
-                                            hue=hue, hue_order=hue_order, x_bin_num=scatter_config["x_bin_num"], ci=scatter_config["ci"], 
-                                            fit_reg=scatter_config["fit_reg"], scatter=scatter_config["scatter"],custom_var=custom_var)
+                                            hue=hue_, hue_order=hue_order, x_bin_num=scatter_config["x_bin_num"], ci=scatter_config["ci"], 
+                                            fit_reg=scatter_config["fit_reg"], scatter=scatter_config["scatter"],custom_var=custom_var,hue_palette=scatter_config['hue_cmap_list'][i])
 
             if 'mean_couple_list' in scatter_config.keys():
                 if len(scatter_config['mean_couple_list']) > 0:
                     if quiet < 2:
                         print("Plotting scatter plots of whole-track parameters...")
-                for param_vs_param in scatter_config['mean_couple_list']:
+                for i,param_vs_param in enumerate(scatter_config['mean_couple_list']):
                     x_param, y_param = param_vs_param
+                    hue_ = scatter_config['mean_hue_var_list'][i] if hue is None else hue  # if hue not already defined by dataset, replace by specific hue
                     tpl.plot_param_vs_param(data_dir, x_param, y_param, df_mean, plot_dir=sub_dir, plot_config=plot_config,
-                                            prefix='track_', hue=hue, hue_order=hue_order, x_bin_num=scatter_config["x_bin_num"], ci=scatter_config["ci"], 
-                                            fit_reg=scatter_config["fit_reg"], scatter=scatter_config["scatter"],custom_var=custom_var)
+                                            prefix='track_', hue=hue_, hue_order=hue_order, x_bin_num=scatter_config["x_bin_num"], ci=scatter_config["ci"], 
+                                            fit_reg=scatter_config["fit_reg"], scatter=scatter_config["scatter"],custom_var=custom_var,hue_palette=scatter_config['mean_hue_cmap_list'][i])
 
         if boxplot_config['run']:
             fn = osp.join(config_dir, 'boxplot_config.csv')
@@ -269,17 +272,19 @@ def traj_analysis(data_dir, data=None, image=None, refresh=False, parallelize=Fa
                 if len(boxplot_config['var_list']) > 0:
                     if quiet < 2:
                         print("Plotting instantaneous parameters boxplots...")
-                for p in boxplot_config['var_list']:
+                for i,p in enumerate(boxplot_config['var_list']):
                     tpl.plot_param_boxplot(data_dir, df=df, x_param=hue, param=p, order=hue_order,save_stat=boxplot_config['save_stat'], boxplot=boxplot_config['boxplot'],
-                                   swarmplot=boxplot_config['swarmplot'], plot_config=plot_config, plot_dir=sub_dir,custom_var=custom_var)
+                                   swarmplot=boxplot_config['swarmplot'], plot_config=plot_config, plot_dir=sub_dir,custom_var=custom_var,
+                                   hue_swarm=boxplot_config['hue_var_list'][i],hue_swarm_palette=boxplot_config['hue_cmap_list'][i])
 
             if 'mean_var_list' in boxplot_config.keys():
                 if len(boxplot_config['mean_var_list']) > 0:
                     if quiet < 2:
                         print("Plotting whole-track boxplots...")
-                for p in boxplot_config['mean_var_list']:
+                for i,p in enumerate(boxplot_config['mean_var_list']):
                     tpl.plot_param_boxplot(data_dir, df=df_mean, x_param=hue, param=p, order=hue_order, save_stat=boxplot_config['mean_save_stat'], boxplot=boxplot_config['mean_boxplot'],
-                                   swarmplot=boxplot_config['mean_swarmplot'], plot_config=plot_config, plot_dir=sub_dir, prefix='track_',custom_var=custom_var)
+                                   swarmplot=boxplot_config['mean_swarmplot'], plot_config=plot_config, plot_dir=sub_dir, prefix='track_',custom_var=custom_var,
+                                   hue_swarm=boxplot_config['mean_hue_var_list'][i],hue_swarm_palette=boxplot_config['mean_hue_cmap_list'][i])
 
 
     return df_list
