@@ -123,13 +123,16 @@ def compare_datasets(data_dir, df_list=[], track_df_list=[], MSD_df_list=[], dat
 
     if MSD_plot_param is not None:
         # add parameters
-        print("plotting MSD")
-        MSD_plot_param['dim'] = 2
-        MSD_plot_param['fitrange'] = False
-        MSD_plot_param['plot_all_MSD'] = True
-        MSD_plot_param['plot_single_MSD'] = False
-        tpl.plot_all_MSD(data_dir, msd_all=pooled_MSD_df, fit_model=None, MSD_parameters=MSD_plot_param, hue=hue,
-                         hue_order=hue_order, plot_config=plot_config, plot_dir=plot_dir)
+        if pooled_MSD_df.shape[0] > 0:
+            print("plotting MSD")
+            MSD_plot_param['dim'] = 2
+            MSD_plot_param['fitrange'] = False
+            MSD_plot_param['plot_all_MSD'] = True
+            MSD_plot_param['plot_single_MSD'] = False
+            tpl.plot_all_MSD(data_dir, msd_all=pooled_MSD_df, fit_model=None, MSD_parameters=MSD_plot_param, hue=hue,
+                             hue_order=hue_order, plot_config=plot_config, plot_dir=plot_dir)
+        else: 
+            print("no MSD data")
 
     return pooled_all_df, pooled_track_df
 
